#!/usr/bin/env python

import pymongo
import os
import argparse
import logging
import time
import multiprocessing as mp
from config import globals
from lib import utils

# Command line parsing
parser = argparse.ArgumentParser(
    description="Dump the tweets of a database to a JSON file")
parser.add_argument("-H", "--host", type=str, default="localhost")
parser.add_argument("-p", "--port", type=int, default=27017)
parser.add_argument("-s", "--pagesize", type=int, default=1000)
parser.add_argument("-v", "--verbose", action="store_true")
parser.add_argument("-t", "--timing", action="store_true")
parser.add_argument("database", type=str)
args = parser.parse_args()

# Logging config
logformat = "[%(asctime)s] %(message)s"
dateformat = "%H:%M:%S"
if args.verbose:
    logging.basicConfig(
        level=logging.DEBUG, format=logformat, datefmt=dateformat)
else:
    logging.basicConfig(
        level=logging.ERROR, format=logformat, datefmt=dateformat)
logger = logging.getLogger(__name__)

# Initialize some variables
script_dir = os.path.dirname(__file__)
output_dir = os.path.join(script_dir, "pymongodump", args.database)
header_file = os.path.join(script_dir, "config", "header.txt")
with open(header_file) as f:
    header = f.readline()
buffer_tweets = {}
task_queue = mp.Queue()
if args.timing:
    globals.timing = True
    time0 = time.time()

# MongoDB connection to get page index
client = pymongo.MongoClient(args.host, args.port)
database_tweets = client[args.database]["tweets"]
page_index = utils.get_page_index(database_tweets, args.pagesize)
client.close()
logger.debug(
    "Database {} partitioned in {} pages of {} tweets (maximum)"
    .format(args.database, len(page_index), args.pagesize))


# Build a picklable function that we can pass to map
def process_data_page(
        page, host=args.host, port=args.port, database=args.database,
        pagesize=args.pagesize, header=header, outputdir=output_dir,
        queue=task_queue):

    utils.process_page(
        host, port, database, header, output_dir, pagesize, page, queue)


try:
    # Launch single process to write to the filesystem
    writer_worker = mp.Process(
        target=utils.filesystem_writer, args=(task_queue, header, ))
    writer_worker.start()

    # Launch pool of workers to perform the format conversion
    with mp.Pool() as pool:
        pool.map(process_data_page, page_index)
    task_queue.put("END")

except Exception as exc:
    logger.error("A fatal error occurred. Script will terminate")
    error_page = exc  # Change this
    utils.dump_recovery_file(
        args.host, args.port, args.database, args.pagesize,
        globals.dumped_pages, error_page, output_dir)


if globals.timing:
    time1 = time.time()
utils.generate_metadata_file(output_dir)
if globals.timing:
    logger.critical(
        "Time spent generating metadata file: {}s"
        .format(time.time() - time1))
    logger.critical(
        "Total execution time: {}s"
        .format(time.time() - time0))
